#include "CBaseDebug.h"
#include "common.h"
#include "utility.h"
#define PROP_STRING TEXT("CBaseDebug")

HFONT CBaseDebug::hFont;
HWND CBaseDebug::hWndDebug;
HWND CBaseDebug::hTabCtrl;

bool CBaseDebug::bIsValid = false;
int CBaseDebug::nReferenceCount = 0;
std::vector<HWND> CBaseDebug::hWndVector;

LRESULT CALLBACK CBaseDebug::WndProc(HWND hWnd , UINT msg , WPARAM wParam , LPARAM lParam)
{
	CBaseDebug *pCBaseDebug;
	pCBaseDebug = (CBaseDebug *)GetProp( hWnd, PROP_STRING );
	if( pCBaseDebug == 0 ){
		return DefWindowProc(hWnd , msg , wParam , lParam);
	}

	switch (msg) {
	case WM_SIZE:
		pCBaseDebug->FixWindowSize( LOWORD(lParam), HIWORD(lParam) );
		break;
	case WM_NOTIFY:
		switch (((NMHDR *)lParam)->code) {
		case TCN_SELCHANGE:
			pCBaseDebug->ChangeTabIndex();
		}
		break;
	case WM_CLOSE:
		CheckMenuItem( g.hMenu, ID_MENU_INFO, MF_UNCHECKED );
		ShowWindow( hWnd, SW_HIDE );
		return 0;
	}
	return DefWindowProc(hWnd , msg , wParam , lParam);
}

int CBaseDebug::AddWindow( HWND hWnd, LPTSTR lpTabString )
{
	int nIndex;
	hWndTab = hWnd;
	nIndex = (int)hWndVector.size();
	hWndVector.push_back( hWnd );
	AddTabCtrl( nIndex, lpTabString );
	return nIndex;
}

bool CBaseDebug::FixWindowSize( int nWidth, int nHeight )
{
	MoveWindow( hTabCtrl, 0, 0, nWidth, DW_TAB_HEIGHT, true );
	for( UINT i = 0 ; i < hWndVector.size() ; i++ ){
		MoveWindow( hWndVector[i], 0, DW_TAB_HEIGHT, nWidth, nHeight - DW_TAB_HEIGHT , true );
	}
	return 0;
}

bool CBaseDebug::SetFont()
{
	hFont = CreateFont(
		14 , 0 , 0 , 0 , 100 , false, false , false ,
		SHIFTJIS_CHARSET , OUT_DEFAULT_PRECIS ,
		CLIP_DEFAULT_PRECIS , DEFAULT_QUALITY , 
		VARIABLE_PITCH | FF_ROMAN , TEXT("Tahoma")
	);
	SendMessage( hTabCtrl, WM_SETFONT, (WPARAM)hFont, true );
	return true;
}

bool CBaseDebug::CBaseDebug_CreateWindow()
{
	if( hWndDebug != NULL ) return true;

	WNDCLASS winc;
	HINSTANCE hInstance = GetModuleHandle(NULL);

	winc.style			= CS_HREDRAW | CS_VREDRAW;
	winc.lpfnWndProc	= CBaseDebug::WndProc;
	winc.cbClsExtra		= winc.cbWndExtra	= 0;
	winc.hInstance		= hInstance;
	winc.hIcon			= LoadIcon( hInstance, "ICON_MAIN" );
	winc.hCursor		= LoadCursor(NULL , IDC_ARROW);
	winc.hbrBackground	= (HBRUSH)GetStockObject(WHITE_BRUSH);
	winc.lpszMenuName	= NULL;
	winc.lpszClassName	= TEXT("CBaseDebug");

	if (!RegisterClass(&winc)) return false;

	hWndDebug = CreateWindowEx(
			0, 
			TEXT("CBaseDebug") , TEXT("EBhE") ,
			WS_OVERLAPPEDWINDOW ,
			0 , 0 , DW_DEFAULT_WIDTH , DW_DEFAULT_HEIGHT , NULL , NULL ,
			hInstance , NULL
	);
	if (hWndDebug == NULL) return false;

	SetProp( hWndDebug, PROP_STRING, (HANDLE)this );

	return true;
}

bool CBaseDebug::CBaseDebug_CreateTabCtrl()
{
	if( hTabCtrl != NULL ) return true;
	InitCommonControls();
	hTabCtrl = CreateWindowEx( 0 , WC_TABCONTROL , NULL ,
		WS_CHILD | WS_CLIPSIBLINGS | WS_VISIBLE ,
		0 , 0 , DW_DEFAULT_WIDTH , DW_TAB_HEIGHT , hWndDebug , (HMENU)0x10 ,
		GetModuleHandle(NULL) , NULL
	);
	if( hTabCtrl == NULL ) return false;
	return true;
}

bool CBaseDebug::AddTabCtrl( int nIndex, LPTSTR lpTabString )
{
	TCITEM tcItem;
	tcItem.mask = TCIF_TEXT;
	tcItem.pszText = lpTabString;
	TabCtrl_InsertItem( hTabCtrl , nIndex, &tcItem );
	return true;
}

bool CBaseDebug::ChangeTabIndex( void )
{
	static HWND hWndBeforeActive;
	if( hWndVector.size() ){
		HWND hWndNowActive = hWndVector[TabCtrl_GetCurSel(hTabCtrl)];
		ShowWindow( hWndBeforeActive, SW_HIDE );
		ShowWindow( hWndNowActive, SW_SHOW );
		hWndBeforeActive = hWndNowActive;
	}
	return 0;
}

HWND CBaseDebug::GetDebugWindowHandle( void )
{
	return hWndDebug;
}

CBaseDebug::CBaseDebug()
{
	nReferenceCount++;
	if( bIsValid == true ) return;
	bIsValid = CBaseDebug_CreateWindow();
	if( bIsValid == false ) return;
	bIsValid = CBaseDebug_CreateTabCtrl();
	if( bIsValid == false ) return;
	ChangeTabIndex();
	SetFont();
}

CBaseDebug::~CBaseDebug()
{
	nReferenceCount--;
	if( nReferenceCount == 0 )
	{
		DeleteObject( (HGDIOBJ)hFont );
		RemoveProp( hWndDebug, PROP_STRING );
		PostQuitMessage(0);
	}

	for( UINT i = 0 ; i < hWndVector.size() ; i++ )
	{
		if( hWndVector[i] == hWndTab )
		{
			DestroyWindow( hWndTab );
			hWndVector.erase( hWndVector.begin()+ i );
			if( i == TabCtrl_GetCurSel( hTabCtrl ) )
			{
				if( i != hWndVector.size() )
				{
					TabCtrl_SetCurSel( hTabCtrl, i+1 );
				}
				else
				{
					TabCtrl_SetCurSel( hTabCtrl, i-1 );
				}
				ChangeTabIndex();
			}
			TabCtrl_DeleteItem( hTabCtrl, i );
			break;
		}
	}
}



#undef PROP_STRING