
#include "kakasi_func.h"
#include "ConvReplace.h"
#include "common.h"

static CConvReplace ConvReplaceBefore( "ConvBefore.txt" );
static CConvReplace ConvReplaceAfter( "ConvAfter.txt" );
static CConvReplace ConvReplace( "ConvFilename.txt", "\\=(yen)\n/=(slash)\n:=(colon)\n*=(asterisk)\n?=(question)\n\"=(double quotation)\n<=(left angle bracket)\n>=(right angle bracket)\n|=(pipeline)" );
CConvDatabase ConvDatabase;

char *Kakasi( int argc, char **argv, char *lpInput )
{
	int nRet;
	kakasi_close_kanwadict();
	nRet = kakasi_getopt_argv( argc, argv );
	if( nRet )
	{
		debug << "[G[]kakasi_getopt_argv()" >> 0;
	}
	return kakasi_do( lpInput );
}

char *KakasiRoman( char *lpInput )
{
	std::string sInput;
	char *lpWakati;
	char *lpRoman;

	char *lpArgvWakati[] = { "kakasi", "-isjis", "-w", "-kK" };
	char *lpArgvRoman[] = { "kakasi", "-Ha", "-Ka", "-Ja", "-Ea", "-ka" }; // "-rk"

	lpWakati = Kakasi( 4, lpArgvWakati, lpInput );
	lpRoman = Kakasi( 6, lpArgvRoman, lpWakati );

	kakasi_free( lpWakati );
	return lpRoman;
}

char *KakasiHankakuKatakana( char *lpInput )
{
	std::string sInput;
	char *lpWakati;
	char *lpRoman;

	char *lpArgvWakati[] = { "kakasi", "-isjis", "-w", "-kK" };
	char *lpArgvKatakanaHan[] = { "kakasi", "-Hk", "-Kk", "-Jk", "-Ea" }; // "-rk"

	lpWakati = Kakasi( 4, lpArgvWakati, lpInput );
	lpRoman = Kakasi( 5, lpArgvKatakanaHan, lpWakati );

	kakasi_free( lpWakati );
	return lpRoman;
}

int KakasiEraseExtFrontSpace( std::string &sInput )
{	// ŔgqO̕svȃXy[X菜
	std::string::size_type nPosExtension;
	nPosExtension = sInput.find_last_of( '.' );
	if( nPosExtension != std::string::npos && sInput[nPosExtension-1] == ' ' )
	{
		sInput.erase( nPosExtension-1, 1 );
		return (int)nPosExtension-1;
	}
	return 0;
}

int KakasiSetEnvironmentDictPath( void )
{
	FILE *lpFile;
	std::string sSetEnvVbs;
	char *lpTmp = NULL;
	char *lpSetEnvFilename = "setenv.vbs";
	int nEnvSize;
	std::string sKanwaDictPath;
	std::string sItaijiDictPath;

	// t@C̃pX擾
	sKanwaDictPath = g.lpSoftDir;
	sItaijiDictPath = g.lpSoftDir;
	sKanwaDictPath += "\\share\\kakasi\\kanwadict";
	sItaijiDictPath += "\\share\\kakasi\\itaijidict";

	// TEMPfBNg
	GetTempPath( g.nBufSize, g.lpBuf );
	g.sPathSetEnvTmp = g.lpBuf;
	g.sPathSetEnvTmp += lpSetEnvFilename;

	// ϐo^Ă邩`FbN
	nEnvSize = GetEnvironmentVariable( "KANWADICTPATH", lpTmp, 0 );
	if( nEnvSize )
	{
		GetEnvironmentVariable( "KANWADICTPATH", g.lpBuf, nEnvSize );
		if( sKanwaDictPath == g.lpBuf )
		{
			return 0;
		}
	}

	// ϐ̐ݒ
	sSetEnvVbs = "Set objShell = WScript.CreateObject(\"WScript.Shell\")\nSet colEnv = objShell.Environment(\"Volatile\")\ncolEnv.Item(\"KANWADICTPATH\") = ";
	sSetEnvVbs += "\"";
	sSetEnvVbs += sKanwaDictPath;
	sSetEnvVbs += "\"";
	sSetEnvVbs += "\n";
	sSetEnvVbs += "colEnv.Item(\"ITAIJIDICTPATH\") = ";
	sSetEnvVbs += "\"";
	sSetEnvVbs += sItaijiDictPath;
	sSetEnvVbs += "\"";
	lpFile = fopen( g.sPathSetEnvTmp.c_str(), "w" );
	fputs( sSetEnvVbs.c_str(), lpFile );
	fclose( lpFile );

	ShellExecute( NULL, "open", g.sPathSetEnvTmp.c_str(), NULL, NULL, SW_SHOWNORMAL );

	// ϐݒ
	SetEnvironmentVariable( "KANWADICTPATH", sKanwaDictPath.c_str() );
	SetEnvironmentVariable( "ITAIJIDICTPATH", sItaijiDictPath.c_str() );


	return 1;
}


std::string KakasiWrap( char *lpInput, std::string sFilePath, int nFlag, int *nReturn )
{
	char *lpConv;
	int nMode = KAKASI_MODE_CONV;
	std::string sInput;
	std::string sConv;

	sInput = lpInput;

	if( GetMenuItemCheckState( g.hMenu, ID_MENU_CONV_AUTO ) || GetMenuItemCheckState( g.hMenu, ID_MENU_CONV_CONV ) )
	{

		ConvReplaceBefore.Replace( sInput );

		// [}EpJ^Jiϊ
		if( GetMenuItemCheckState( g.hMenu, ID_MENU_CONV_ROMAN ) == true )
		{
			lpConv = KakasiRoman( const_cast<char *>(sInput.c_str()) );
		}
		else
		{
			lpConv = KakasiHankakuKatakana( const_cast<char *>(sInput.c_str()) );
		}

		sConv = lpConv;

		ConvReplaceAfter.Replace( sConv );

		// t@Cϊ
		if( nFlag == KAKASI_FLAG_FILENAME )
		{
			KakasiEraseExtFrontSpace( sConv );
			ConvReplace.Replace( sConv );
		}

		// f[^x[X֏
		if( lstrcmp( lpInput, sConv.c_str() ) != 0 )
		{
			ConvDatabase.AddData( lpInput, sConv, sFilePath, nFlag );
		}

		kakasi_free( lpConv );
	}
	else
	{
		sConv = lpInput;
	}

	// f[^x[X畜
	if( GetMenuItemCheckState( g.hMenu, ID_MENU_CONV_AUTO ) && lstrcmp( lpInput, sConv.c_str() ) == 0 || GetMenuItemCheckState( g.hMenu, ID_MENU_CONV_RESTORE ) )
	{
		sConv = ConvDatabase.GetData( lpInput, sFilePath, nFlag );
		nMode = KAKASI_MODE_REST;
	}

	if( nReturn != 0 )
	{
		*nReturn = nMode;
	}

	return sConv;
}

CConvDatabase::CConvDatabase( void )
{
	lpDatabaseName = "ConvData.dat";

	sDatabasePath = g.lpSoftDir;
	sDatabasePath += "\\";
	sDatabasePath += lpDatabaseName;

	lpDatabasePath = sDatabasePath.c_str();
}

int CConvDatabase::AddData( std::string sInput, std::string sData, std::string &sPath, int nFlag )
{
	std::string sData2;

	ReplaceMark( sData );
	ReplaceMark( sInput );

	WritePrivateProfileString( "Data", sData.c_str(), sInput.c_str(), lpDatabasePath );

	// t@CȊOȂt@C+lۑ
	if( nFlag == KAKASI_FLAG_DEFAULT )
	{
		GetFileName( sPath, sData2 );
		sData2 = sData2 + std::string("+") + sData;
		ReplaceMark( sData2 );
		WritePrivateProfileString( "Data", sData2.c_str(), sInput.c_str(), lpDatabasePath );
	}

	return true;
}

std::string CConvDatabase::GetData( std::string sInput, std::string &sPath, int nFlag )
{
	std::string sInput2;
	std::string sFilename;
	std::string sGetData = sInput;
	ReplaceMark( sInput );

	// t@CȊÔƂ́At@CČ
	if( nFlag == KAKASI_FLAG_DEFAULT )
	{
		GetFileName( sPath, sFilename );
		ReplaceMark( sFilename );
		GetPrivateProfileString( "Data", sFilename.c_str(), "=", g.lpBuf, g.nBufSize, lpDatabasePath );
		if( lstrcmp( g.lpBuf, "=" ) != 0 )
		{
			// t@Cf[^x[XɌꍇA
			// ɓ{ϊĂ猋Č
			sInput2 = std::string( g.lpBuf ) + std::string("+") + sInput;
		}
		else
		{
			// t@Cf[^x[XɌȂꍇ́Â܂܌Č
			sInput2 = sInput2 + std::string("+") + sInput;
		}

		ReplaceMark( sInput2 );
		GetPrivateProfileString( "Data", sInput2.c_str(), "=", g.lpBuf, g.nBufSize, lpDatabasePath );
		// Ήf[^݂ꍇÃf[^ǂ݂ƂB
		if( lstrcmp( g.lpBuf, "=" ) != 0 )
		{
			sGetData = g.lpBuf;
			ReplaceOrg( sGetData );
			return sGetData;
		}
		else
		{
			// [t@CɑΉ^Ôݕ]Ƀ`FbNĂ΁AreturnB
			// tĂȂꍇ́At@CɊ֌WȂf[^x[X當
			if( GetMenuItemCheckState( g.hMenu, ID_MENU_CONV_TAG_AGREE ) ) return sInput;
		}
	}

	GetPrivateProfileString( "Data", sInput.c_str(), "=", g.lpBuf, g.nBufSize, lpDatabasePath );
	if( lstrcmp( g.lpBuf, "=" ) != 0 )
	{
		sGetData = g.lpBuf;
		sGetData = sGetData;
		ReplaceOrg( sGetData );
	}

	return sGetData;
}

void CConvDatabase::ReplaceMark( std::string &sData )
{
	replace_mb( sData, '=', "%__EQUAL__%" );
	replace_mb( sData, '\r', "%__CR__%" );
	replace_mb( sData, '\n', "%__LF__%" );
	replace_mb( sData, '[', "%__LEFT_BRACKET__%" );
	replace_mb( sData, ']', "%__RIGHT_BRACKET__%" );
	/*
	replace( sData, "=", "%__EQUAL__%" );
	replace( sData, "\r", "%__CR__%" );
	replace( sData, "\n", "%__LF__%" );
	replace( sData, "[", "%__LEFT_BRACKET__%" );
	replace( sData, "]", "%__RIGHT_BRACKET__%" );
	*/
}

void CConvDatabase::ReplaceOrg( std::string &sData )
{
	replace( sData, "%__EQUAL__%", "=" );
	replace( sData, "%__CR__%", "\r" );
	replace( sData, "%__LF__%", "\n" );
	replace( sData, "%__LEFT_BRACKET__%", "[" );
	replace( sData, "%__RIGHT_BRACKET__%", "]" );
}
