#include "utility.h"

int GetWindowPosX( HWND hWnd )
{
	RECT Rect;
	GetWindowRect( hWnd, &Rect );
	return Rect.left;
}

int GetWindowPosY( HWND hWnd )
{
	RECT Rect;
	GetWindowRect( hWnd, &Rect );
	return Rect.top;
}

int GetWindowWidth( HWND hWnd )
{
	RECT Rect;
	GetWindowRect( hWnd, &Rect );
	return Rect.right - Rect.left;
}

int GetWindowHeight( HWND hWnd )
{
	RECT Rect;
	GetWindowRect( hWnd, &Rect );
	return Rect.bottom - Rect.top;
}

BOOL SetWindowPosition( HWND hWnd, int X, int Y )
{
	return SetWindowPos( hWnd, 0, X, Y, 0, 0, SWP_NOSIZE | SWP_NOZORDER );
}

BOOL SetWindowSize( HWND hWnd, int X, int Y )
{
	return SetWindowPos( hWnd, 0, 0, 0, X, Y, SWP_NOMOVE | SWP_NOZORDER );
}

BOOL SetWindowZOrder( HWND hWnd, HWND hWndInsertAfter )
{
	return SetWindowPos( hWnd, hWndInsertAfter, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE );
}

LRESULT SetDefaultGUIFont( HWND hWnd )
{
	HFONT hFont;
	hFont = (HFONT)GetStockObject( DEFAULT_GUI_FONT );
	return SendMessage( hWnd, WM_SETFONT, (WPARAM)hFont, true );
}

bool GetMenuItemCheckState( HMENU hMenu, UINT uItem )
{
	MENUITEMINFO mii;
	mii.cbSize = sizeof( MENUITEMINFO );
	mii.fMask = MIIM_STATE;
	if( GetMenuItemInfo( hMenu, uItem, false, &mii) == 0 ) return false;
	if( mii.fState & MFS_CHECKED )
	{
		return true;
	}
	else
	{
		return false;
	}
}

DWORD ReplaceCheckMenu( HMENU hMenu, UINT uItem )
{
	if( GetMenuItemCheckState( hMenu, uItem ) == true )
	{
		return CheckMenuItem( hMenu, uItem, MF_UNCHECKED );
	}
	else
	{
		return CheckMenuItem( hMenu, uItem, MF_CHECKED );
	}
}

std::string & GetFileName( std::string &sFilePath, std::string &sFilename )
{
	return GetFileName( sFilePath.c_str(), sFilename );
}

std::string & GetFileName( const char *lpFilePath, std::string &sFilename )
{
	char drive[_MAX_DRIVE];
	char dir[_MAX_DIR];
	char fname[_MAX_FNAME];
	char ext[_MAX_EXT];
	_splitpath( lpFilePath, drive, dir, fname, ext );
	sFilename = fname;
	sFilename += ext;
	return sFilename;
}

std::string & GetFileDir( std::string &sFilePath, std::string &sFileDir )
{
	return GetFileDir( sFilePath.c_str(), sFileDir );
}

std::string & GetFileDir( const char *lpFilePath, std::string &sFileDir )
{
	char drive[_MAX_DRIVE];
	char dir[_MAX_DIR];
	char fname[_MAX_FNAME];
	char ext[_MAX_EXT];
	_splitpath( lpFilePath, drive, dir, fname, ext );
	sFileDir = drive;
	sFileDir += dir;
	return sFileDir;
}

std::string & GetFileExt( std::string &sFilePath, std::string &sFileExt )
{
	return GetFileExt( sFilePath.c_str(), sFileExt );
}

std::string & GetFileExt( const char *lpFilePath, std::string &sFileExt )
{
	char drive[_MAX_DRIVE];
	char dir[_MAX_DIR];
	char fname[_MAX_FNAME];
	char ext[_MAX_EXT];
	_splitpath( lpFilePath, drive, dir, fname, ext );
	sFileExt = ext;
	return sFileExt;
}

// QURL : http://www.alter.org.ua/docs/win/args/index.php?lang=en&
static CHAR** CommandLineToArgvA( CHAR *lpCmdLine, int *_argc )
{
	CHAR **argv;
	CHAR *_argv;
	ULONG len;
	ULONG argc;
	CHAR c;
	ULONG i, j;

	bool bFlagInQM;
	bool bFlagInText;
	bool bFlagInSpace;

	len = (ULONG)strlen( lpCmdLine );
	i = ( ( len + 2 ) / 2 ) * sizeof PVOID + sizeof PVOID;

	argv = (PCHAR*)GlobalAlloc( GMEM_FIXED, i + ( len + 2 ) * sizeof CHAR );
	_argv = (PCHAR)( ( (PUCHAR)argv ) + i );

	argc = 0;
	argv[argc] = _argv;
	bFlagInQM = false;
	bFlagInText = false;
	bFlagInSpace = true;
	i = 0;
	j = 0;

	while( c = lpCmdLine[i] )
	{
		if( bFlagInQM )
		{
			if( c == '\"' )
			{
				bFlagInQM = false;
			} else {
				_argv[j] = c;
				j++;
			}
		}
		else
		{
			switch( c )
			{
			case '\"':
				bFlagInQM = true;
				bFlagInText = true;
				if( bFlagInSpace )
				{
					argv[argc] = _argv + j;
					argc++;
				}
				bFlagInSpace = false;
				break;

			case ' ':
			case '\t':
			case '\n':
			case '\r':
				if( bFlagInText )
				{
					_argv[j] = '\0';
					j++;
				}
				bFlagInText = false;
				bFlagInSpace = true;
				break;

			default:
				bFlagInText = true;
				if( bFlagInSpace )
				{
					argv[argc] = _argv + j;
					argc++;
				}
				_argv[j] = c;
				j++;
				bFlagInSpace = false;
				break;
			}
		}
		i++;
	}
	_argv[j] = '\0';
	argv[argc] = NULL;

	*_argc = argc;
	return argv;
}

std::vector<std::string> GetCommandLineArgs( bool bFlagIncludeSelfExePath )
{
	int nLoopStart = 0;
	if( !bFlagIncludeSelfExePath )
	{
		nLoopStart = 1;
	}

	int nNumArgs;
	LPSTR *lpCommandLine;
	lpCommandLine = CommandLineToArgvA( GetCommandLine(), &nNumArgs );
	std::vector<std::string> vsReturnArgs;
	vsReturnArgs.reserve( nNumArgs );
	for( int i = nLoopStart ; i < nNumArgs ; i++ )
	{
		vsReturnArgs.push_back( lpCommandLine[i] );
	}

	LocalFree( lpCommandLine );
	return vsReturnArgs;
}

std::string GetExePath( void )
{
	static std::vector<std::string> vsCommandLine = GetCommandLineArgs( true );
	return vsCommandLine[0];
}

std::string GetExeDir( void )
{
	std::string sExeDir;
	GetFileDir( GetExePath().c_str(), sExeDir );
	return sExeDir;
}

bool CheckCommandLineOption( const std::vector<std::string> &vsCommandLine, const std::string &sCheckFlag )
{
	for( unsigned i = 0 ; i < vsCommandLine.size() ; ++i )
	{
		if( sCheckFlag == vsCommandLine[i] )
		{
			return true;
		}
	}
	return false;
}